'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fetch = _interopDefault(require('isomorphic-fetch'));

function serialize(value) {
    let serialized = null;
    try {
        serialized = JSON.stringify(value);
    }
    catch (error) { }
    return serialized;
}
function isPlainObject(value) {
    return typeof value === "object" && value.toString() === "[object Object]";
}
function sendUnserializable(holeId) {
    return fetch("http://localhost:17341/unserializable", {
        method: "POST",
        mode: "cors",
        body: JSON.stringify({
            id: holeId,
        }),
        headers: {
            "Content-Type": "application/json",
        },
    }).catch((err) => console.log(err.message));
}
// This is here so that multiple types would not be edited simultaniously
let sampleQueue = Promise.resolve();
function sendSample(holeId, input) {
    sampleQueue = sampleQueue.then(() => fetch("http://localhost:17341/samples", {
        method: "POST",
        mode: "cors",
        body: JSON.stringify({
            id: holeId,
            sample: input,
        }),
        headers: {
            "Content-Type": "application/json",
        },
    }).catch((err) => console.log(err.message)));
    return sampleQueue;
}
const debounce = (func, waitFor) => {
    let timeout;
    return (...args) => new Promise((resolve) => {
        if (timeout) {
            clearTimeout(timeout);
        }
        timeout = setTimeout(() => resolve(func(...args)), waitFor);
    });
};
function typeholeFactory(id) {
    const emitSample = debounce(sendSample, 300);
    let previousValue = null;
    return function typehole(input) {
        const serialized = serialize(input);
        if (serialized === previousValue) {
            return input;
        }
        previousValue = serialized;
        if (!serialized || (serialized === "{}" && !isPlainObject(input))) {
            sendUnserializable(id);
        }
        else {
            emitSample(id, input);
        }
        return input;
    };
}
var index = new Proxy({}, {
    get: function (target, prop, receiver) {
        if (/^t([0-9]+)?$/.test(prop.toString())) {
            return typeholeFactory(prop);
        }
        return Reflect.get(target, prop, receiver);
    },
});

module.exports = index;
